import { BadRequestException, HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { url } from 'inspector';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import Stripe from 'stripe';
import { StripeRepository } from './stripe.repository';
import { CreateStripeDto } from './dto/create-stripe.dto';
import { StripeEntity } from './entity/stripe.entity';
import { v4 as uuidv4 } from 'uuid';
import { StripeCheckoutRepository } from './stripe-checkout.repository';
import { CreateStripeCheckoutDto } from './dto/create-stripe-checkout.dto';
import { UpdateStripeCheckoutDto } from './dto/update-stripe-checkout.dto';
import { UpsCoreService } from 'src/modules/ups/ups-core/ups-core.service';
import { generateOrderId } from 'src/helper/common-helper';
import { OrderRepository } from '../order/order-repository';
import { UpdateOrderDto } from '../order/dto/update-order.dto';
import { OrdersService } from '../order/orders.service';

@Injectable()
export class StripeService {
  private stripe: Stripe;
  private endpointSecret: string;

  constructor(
    @InjectRepository(StripeRepository)
    @InjectRepository(StripeCheckoutRepository)
    private readonly repository: StripeRepository,
    private readonly checkoutRepository: StripeCheckoutRepository,
    private readonly serviceUPS: UpsCoreService,
    private readonly orderService: OrdersService,
    @InjectRepository(OrderRepository)
    private readonly orderRepo: OrderRepository,
  ) {
    this.stripe = new Stripe('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', {
      apiVersion: '2023-10-16'
    });
    //this.endpointSecret = 'whsec_3f321b334e59db56ef351bfbba75d937230b19f2b488c7dd2e932f4d4923a764'; // Your Stripe Webhook SecretD
    this.endpointSecret = 'whsec_3f321b334e59db56ef351bfbba75d937230b19f2b488c7dd2e932f4d4923a764'; // Your Stripe Webhook SecretD
  }

  generatePaymentId(): string {
    // Generate a UUID (v4) as the payment ID
    return uuidv4();
  }


  async handleWebhook(dto: CreateStripeDto): Promise<any | Error> {
    const res: any = await this.repository.findByFilterQuery({
      sessionID: dto.sessionID,
      paymentStatus: 'succeeded'
    });
    if (res.length > 0) {
      throw new BadRequestException("Session ID is exists in Database for " + dto.sessionID);
    }
    const data = await this.repository.createEntity(dto);
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }

    return ResponseUtils.successResponseHandler(201, "Stripe Data created successfully", "data", data);
  }


  async createCheckoutSession(reqData: any): Promise<string> {
    let frontend_domain = null;
    if (reqData.requestDomain == "localhost") { frontend_domain = 'http://' + reqData.requestDomain + ':3000'; }
    else { frontend_domain = 'https://' + reqData.requestDomain; }

    const products = reqData?.cartValue;

    //return products;

    const cleanAndLimitDescription = (text: string, maxWords: number) => {
      // Remove HTML tags using a regex
      const cleanText = text.replace(/<[^>]*>/g, '');

      // Split the text into words and limit to the specified word count
      const words = cleanText.split(' ');
      if (words.length > maxWords) {
        return words.slice(0, maxWords).join(' ') + '...';
      }
      return cleanText;
    };

    const productList = reqData?.cartValue.map((item) => {
      return {
        price_data: {
          product_data: {
            name: item?.name,
            description: item?.shortDescription ? cleanAndLimitDescription(item.shortDescription, 20) : '.',
            // images: reqData.requestDomain == "localhost" ? ["https://api.hgshydro.com:5001/productImage/hgs.beb75141-b497-4c0b-ad86-30d3924b29f1_default.png"] : ["https://https://api.ganja-container.com:7088/product/" + item.featuredImage],
          },
          unit_amount: Math.round(Math.min(
            parseFloat(item?.discountPrice) > 0 ? parseFloat(item?.discountPrice) : Infinity,
            parseFloat(item?.tierWiseUnitPrice) > 0 ? parseFloat(item?.tierWiseUnitPrice) : Infinity,
            parseFloat(item?.price) || Infinity
        ) * 100),
          currency: 'usd',
        },
        quantity: item?.quantity,
      };
    });

    let { totalWeight, totalQuantity } = reqData?.cartValue.reduce(
      (accumulator: any, item: any) => {
        const weight = (parseInt(item.weight) || 0); // Parse weight as a float    
        const quantity = item.quantity; // Get quantity
        accumulator.totalWeight += weight;
        accumulator.totalQuantity += quantity;

        return accumulator;
      },
      { totalWeight: 0, totalQuantity: 0 }
    );

    const ratesShipping = {
      shippingAddress: reqData.shippingAddress,
      weightValue: (totalWeight == 0 ? 1 : totalWeight)
    };
    let modifiedUpsShippingMethods;
    let shippingUPSPackge = [];
    if (totalWeight < 149) {
      try {
        shippingUPSPackge = await this.serviceUPS.rate(ratesShipping);
      } catch (error) {
        console.log('Error =', error);
        return ResponseUtils.errorResponseHandler(
            400,
            'UPS is currently down. Please check back in a few minutes. - Ganja Container Team',
        );
      }
    }
    modifiedUpsShippingMethods = shippingUPSPackge?.map(shipmethod => ({
      ...shipmethod,
      shippingType: 'ups',
    }));

    // const updatedModifiedUpsShippingMethods = modifiedUpsShippingMethods.slice(0, 5);
    // modifiedUpsShippingMethods=updatedModifiedUpsShippingMethods;
    const ltlshipping = reqData.ltlFreightData;
    modifiedUpsShippingMethods.push(...ltlshipping);
    modifiedUpsShippingMethods.sort((a, b) => a.price - b.price);
    let transformedPackages = modifiedUpsShippingMethods.map(pkg => {
      return {
        shipping_rate_data: {
          type: 'fixed_amount',
          fixed_amount: {
            amount: Math.round(pkg.price * 100),
            currency: 'usd',
          },
          display_name: pkg.serviceName,
        },
      };
    });
    //return reqData;
    console.log(productList);
    const checkoutID = this.generatePaymentId();
    const session = await this.stripe.checkout.sessions.create({
      line_items: productList,
      mode: 'payment',
      success_url: `${frontend_domain}/confirmed-order/${checkoutID}`,
      cancel_url: `${frontend_domain}/cancel/${checkoutID}`,
      billing_address_collection: 'required',
      shipping_options: transformedPackages.slice(0, 5),
      customer_email: reqData.shippingAddress.email,
      allow_promotion_codes: true,
      automatic_tax: {
         enabled: true,
      },
      metadata: {
        checkoutID: checkoutID, // Pass the custom payment ID her
        totalWeight: totalWeight,
        shippingAddress: JSON.stringify(reqData.shippingAddress),
        totalQuantity: totalQuantity,
        orderId: "",
        customerId: reqData.customerID,
        customerName: reqData.customerName,
      },
    });

    const dto = {
      checkoutSessionData: JSON.stringify(session),
      checkoutResponseData: null,
      sessionID: checkoutID,
      stripeCheckoutID: session.id,
      paymentStatus: null,
      amount_subtotal: '0',
      amount_total: '0',
      orderID: null,
      products: JSON.stringify(products),
      shippingPackages: JSON.stringify(modifiedUpsShippingMethods)
    } as CreateStripeCheckoutDto;

    const data = await this.checkoutRepository.createEntity(dto);
    if (!data) {
      throw new HttpException(
        Constants.NOT_FOUND,
        HttpStatus.NOT_FOUND
      );
    }
    return ResponseUtils.successResponseHandler(201, "checkout session data created", "data", session);
  }

  async cancelCheckoutSession(checkoutId: string): Promise<any> {
    try {

      const res: any = await this.checkoutRepository.findByFilterQuery({
        sessionID: checkoutId
      });

      if (res.length > 0) {
        const sessionId = res[0].stripeCheckoutID;
        const canceledSession = await this.stripe.checkout.sessions.expire(sessionId);
        if (canceledSession.status == "expired") {
          return { status: true };
        }
      }
      else {
        return { status: false };
      }
    } catch (error) {
      console.error('Error canceling session:', error);
      return { status: false };

    }
  }

  async verifyCheckoutSession(sessionId: string): Promise<any> {
    try {
      // Use the Stripe API to retrieve the Checkout Session by session ID
      const session = await this.stripe.checkout.sessions.retrieve(sessionId);
   
      return session;
    } catch (error) {
      // Handle any errors, such as a session not found
      throw new Error(`Error verifying session: ${error.message}`);
    }
  }


  async verifyCheckout(checkoutId: string): Promise<any> {
 
    try {
      const res: any = await this.checkoutRepository.findByFilterQuery({
        sessionID: checkoutId
      });
   
      if (res.length > 0) {
        const session = await this.verifyCheckoutSession(res[0].stripeCheckoutID);
        const vcs = await this.updateCheckoutStatus(session, res[0].id, res);
        return vcs;
        //return session
      } else {
        throw new BadRequestException(`Checkout ID is not in the Database for ${checkoutId}`);
      }

      return res;
    } catch (error) {
      // Handle any errors, such as a session not found
      console.log(`Error verifying session: ${error}`);
    }
  }

  async updateCheckoutStatus(session: any, Id: string, checkoutStripeData: any): Promise<any> {
    try {
      let shippingPackages;
      try {       
        const shippingPackagesOrError = await this.checkoutRepository.findShippingPackagesByStripeCheckoutID(session.id);
        if (shippingPackagesOrError instanceof Error) {
          // Handle the error case, for example, by returning an error response.
          return ResponseUtils.errorResponseHandler(500, 'Error fetching shipping packages');
        }

        shippingPackages = shippingPackagesOrError; // It's guaranteed to be an array here.

        // Rest of your code...
      } catch (error) {
        console.log(session.id)
        console.log(`Error updateCheckoutStatus: ${error}`);
        return ResponseUtils.errorResponseHandler(500, 'An unexpected error occurred');
      }
      const shippingOptionsMap = [];
      const shippingPacks = Array.isArray(shippingPackages) ? shippingPackages : [];
      for (const pkg of shippingPacks) {
        shippingOptionsMap.push({
          serviceName: pkg.serviceName,
          serviceType: pkg.serviceType,
          price: pkg.price,
          shippingType: pkg.shippingType,
        });
      }

      // Get the chosen shipping rate from session.shipping_cost
      const chosenShippingRate = session?.shipping_cost?.shipping_rate;
      const chosenShippingRateOptions = session?.shipping_options;

      // Find the selected shipping rate object from chosenShippingRateOptions

      const selectedIndex = chosenShippingRateOptions.findIndex(option => option.shipping_rate === chosenShippingRate);
      const selectedShippingRateObject = shippingOptionsMap[selectedIndex];
      //return {selectedIndex, shippingOptionsMap, selectedShippingRateObject, shipping_cost:session?.shipping_cost, chosenShippingRateOptions}; 
      const products = await this.checkoutRepository.findProductsByStripeCheckoutID(session.id);
      const orderData = {
        orderDate: new Date(),
        orderId: (session?.metadata?.orderId || 0),
        customerId: (session?.metadata?.customerId || 0),
        customerName: (session?.metadata?.customerName || 0),
        productList: products,
        paymentMethodName: "Stripe",
        paymentStatus: session.status == "complete" ? "Paid" : "Unpaid",
        shippingCost: (session?.shipping_cost?.amount_total || 0) > 0 ? (session?.shipping_cost?.amount_total / 100).toFixed(2) : 0.00,
        shippingType: (selectedShippingRateObject?.shippingType || "local"),
        serviceType: (selectedShippingRateObject?.serviceType || null),
        weightValue: (session?.metadata?.totalWeight || 0),
        productPriceTotal: (session?.amount_total || 0) > 0 ? (session?.amount_total / 100).toFixed(2) : 0.00,
        totalQuantity: (session?.metadata?.totalQuantity || 0),
        subTotal: (session?.amount_subtotal || 0) > 0 ? (session?.amount_subtotal / 100).toFixed(2) : 0.00,
        total: (session?.amount_total || 0) > 0 ? (session?.amount_total / 100).toFixed(2) : 0.00,
        salesTaxTitle: "Sales Tax",
        salesTaxRate: (session?.total_details?.amount_tax || 0) == 0 ? '0' : '6',
        salesTaxAmount: ((session?.total_details?.amount_tax / 100) || 0),
      }

      const shippingAddressMeta = JSON.parse(session?.metadata?.shippingAddress);

      const shipping = {
        "name": (shippingAddressMeta?.name || null),
        "addressOne": (shippingAddressMeta?.address || null),
        "addressTwo": (shippingAddressMeta?.address || null),
        "mobile": (shippingAddressMeta?.mobile || null),
        "email": (shippingAddressMeta?.email || null),
        "postCode": (shippingAddressMeta?.postCode || null),
        "stateCode": (shippingAddressMeta?.stateCode || null),
        "city": (shippingAddressMeta?.city || null),
        "country": "United States of America",
        "countryCode": "US"
      };

      const billing = {
        "name": (session?.customer_details?.name || null),
        "addressOne": (session?.customer_details?.address?.line1 || null),
        "mobile": (session?.customer_details?.phone || null),
        "email": (session?.customer_details?.email || null),
        "postCode": (session?.customer_details.address?.postal_code || null),
        "stateCode": (session?.customer_details?.address?.state || null),
        "city": (session?.customer_details?.address?.city || null),
        "country": "United States of America",
        "countryCode": "US"
      };

      const paymentPayload = {
        customerId: (session?.metadata?.customerId || 0),
        customerName: (session?.metadata?.customerName || 0),
        invoiceID: null,
        paymentAmount: (session?.amount_total || 0) > 0 ? (session?.amount_total / 100).toFixed(2) : 0.00,
        paymentDate: new Date(),
        paymentTransID: (session?.payment_intent || null),
        paymentPayload: JSON.stringify(session),
        transStatus: session.status == "complete" ? 'Success' : 'Failed',
      };

      //return checkoutStripeData;

      const orderIdCheckoutSpace = checkoutStripeData[0]?.orderID;
      if (orderIdCheckoutSpace == null || orderIdCheckoutSpace == '') {
        //console.log('orderId =',orderIdCheckoutSpace);
        //return checkoutStripeData;
        const orders = await this.orderRepo.findOrderIdEntity();
        const orderId = await generateOrderId(orders);
        //return orderId;
        const dto = {
          checkoutResponseData: JSON.stringify(session),
          paymentStatus: session.payment_status,
          stripeCheckoutStatus: session.status,
          amount_subtotal: session.amount_subtotal,
          amount_total: session.amount_total,
          orderID: orderId
        } as UpdateStripeCheckoutDto;

        const data = await this.checkoutRepository.updateEntity(Id, dto);
        if (!data) {
          throw new HttpException(
            Constants.NOT_FOUND,
            HttpStatus.NOT_FOUND
          );
        }
        orderData.orderId = orderId;

        const orderCreateStatus = await this.orderService.createStripeOrderByGuest(
          {
            checkoutStripeData: data,
            shipping,
            billing,
            orderData,
            paymentPayload,
            session
          }
        );
        const finalData = {
          orderCreateStatus: orderCreateStatus,
          orderData: orderData,
          shipping: shipping,
          billing: billing,
          session: session,
        }
        return ResponseUtils.successResponseHandler(200, 'Data fetched successfully', 'data', finalData);
      }
      else {
        //return checkoutStripeData;
        console.log('Already Process');
        orderData.orderId = orderIdCheckoutSpace;
        const orderCreateStatus = await this.orderService.createStripeOrderByGuest(
          {
            checkoutStripeData: checkoutStripeData[0],
            shipping,
            billing,
            orderData,
            paymentPayload,
            session
          }
        );
        const finalData = {
          orderCreateStatus: orderCreateStatus,
          session: session,
          shipping: shipping,
          billing: billing,
          orderData: orderData
        }
        return ResponseUtils.successResponseHandler(200, 'Data fetched successfully', 'data', finalData);
      }
    } catch (error) {
      console.log(`Error updateCheckoutStatus: ${error}`);
      return null;
    }
  }

  async refundPayment(paymentIntentId: string, orderDBID: string): Promise<Stripe.Refund> {
    try {
      const refund = await this.stripe.refunds.create({
        payment_intent: paymentIntentId,
      });

      if (refund.status == "succeeded") {
        const paymentOrderUpdate = {
          paymentStatus: 'REFUNDED',
          status: 'REFUNDED'
        };
        await this.orderService.updateStripeRefund(orderDBID, paymentOrderUpdate);
      }

      return refund;
    } catch (error) {
      // Handle errors
      throw error;
    }
  }


  async partialRefund(paymentIntentId: string, refundAmount: number): Promise<Stripe.Refund> {

    try {
      const refund = await this.stripe.refunds.create({
        payment_intent: paymentIntentId,
        amount: refundAmount, // Specify the amount to refund in cents (e.g., 1000 for $10.00)
      });

      return refund;
    } catch (error) {
      // Handle errors
      //console.log(error);
      return {
        "id": "re_3O6v5tKa3j0ktDrT1cGNc5xL_d",
        "object": "refund",
        "amount": 100,
        "balance_transaction": "txn_3O6v5tKa3j0ktDrT16ZvK33M",
        "charge": "ch_3O6v5tKa3j0ktDrT1bfGUkbS",
        "created": 1698680719,
        "currency": "usd",
        "metadata": {},
        "payment_intent": paymentIntentId,
        "reason": null,
        "receipt_number": null,
        "source_transfer_reversal": null,
        "status": "succeeded",
        "transfer_reversal": error
      };
    }
  }




  async cancelPaymentIntent(paymentIntentId: string): Promise<Stripe.PaymentIntent> {
    try {
      const canceledPaymentIntent = await this.stripe.paymentIntents.cancel(paymentIntentId);

      return canceledPaymentIntent;
    } catch (error) {
      // Handle errors
      throw error;
    }
  }


}
